from __future__ import unicode_literals
from . import config
from os.path import basename

import wac


class User(config.Resource):
    type = "users"
    uri_gen = wac.URIGen("users", "")

    def create_application(self, application):
        attrs = application.__dict__.copy()
        return self.applications.create(attrs)


class Application(config.Resource):
    type = "applications"
    uri_gen = wac.URIGen("applications", "")

    def create_partner_user(self, partner):
        attrs = partner.__dict__.copy()
        return self.users.create(attrs)

    def create_processor(self, processor):
        attrs = processor.__dict__.copy()
        return self.processors.create(attrs)


class Processor(config.Resource):
    type = "processors"


class Merchant(config.Resource):
    type = "merchants"
    uri_gen = wac.URIGen("merchants", "")


class Identity(config.Resource):
    type = "identities"
    uri_gen = wac.URIGen("identities", "")

    def provision_merchant_on(self, merchant):
        attrs = merchant.__dict__.copy()
        return self.merchants.create(attrs)

    def create_payment_instrument(self, payment_instrument):
        attrs = payment_instrument.__dict__.copy()
        attrs["identity"] = self.id
        return self.payment_instruments.create(attrs)

    def create_settlement(self, settlement):
        attrs = settlement.__dict__.copy()
        return self.settlements.create(attrs)


class PaymentInstrument(config.Resource):
    type = "payment_instruments"
    uri_gen = wac.URIGen("payment_instruments", "")


class PaymentCard(PaymentInstrument):
    def __init__(self, **kwargs):
        super(PaymentInstrument, self).__init__(**kwargs)
        self.type = "PAYMENT_CARD"


class BankAccount(PaymentInstrument):
    def __init__(self, **kwargs):
        super(PaymentInstrument, self).__init__(**kwargs)
        self.type = "BANK_ACCOUNT"


class Transfer(config.Resource):
    type = "transfers"
    uri_gen = wac.URIGen("transfers", "")

    def reverse(self, refund_amount):
        refund = Refund(refund_amount=refund_amount)
        attrs = refund.__dict__.copy()
        return self.reversals.create(attrs)


class Refund(config.Resource):
    type = "reversals"


class Authorization(config.Resource):
    type = "authorizations"
    uri_gen = wac.URIGen("authorizations", "")

    def set_void_me(self, void_me):
        self.void_me = void_me
        return self.save()

    def capture(self, amount):
        self.capture_amount = amount
        return self.save()


class Webhook(config.Resource):
    type = "webhooks"
    uri_gen = wac.URIGen("webhooks", "")


class Settlement(config.Resource):
    type = "settlements"
    uri_gen = wac.URIGen("settlements", "")


class Verification(config.Resource):
    type = "verifications"
    uri_gen = wac.URIGen("verifications", "")


class Dispute(config.Resource):
    type = "disputes"

    def upload_evidence(self, evidence_file_path):
        files = {'file': (basename(evidence_file_path), open(evidence_file_path, 'rb'), {'Expires': '0'})}
        return self.evidence.create(files=files)


class Evidence(config.Resource):
    type = "evidence"



from __future__ import unicode_literals
from . import wac_config


def to_uri(href):
    href = href.replace(wac_config.root_url, "") if href else ""
    return "/" + "/".join(filter(None, href.split("/")))



from __future__ import unicode_literals
from coreapi.codecs import HALCodec
from . import API_ROOT
from . import utils
from . import wac_config

import wac
import simplejson as json


def configure(root_url=API_ROOT, **kwargs):
    kwargs['headers'] = {} if 'headers' not in kwargs else kwargs['headers']
    kwargs['headers']['Content-Type'] = 'application/hal+json'
    if 'error_cls' not in kwargs:
        kwargs['error_cls'] = convert_error
    root_url = wac_config.root_url if root_url is None else root_url
    wac_config.reset(root_url, **kwargs)


def convert_error(ex):
    if not hasattr(ex.response, 'data'):
        return ex
    return ResourceErrors(**ex.response.data)


class ObjectifyMixin(wac._ObjectifyMixin):
    def _objectify(self, resource_cls, **fields):
        doc = fields["hal_codec"] if "hal_codec" in fields else HALCodec().load(json.dumps(fields))
        setattr(self, 'uri', utils.to_uri(doc.url) if doc.url else "")
        [setattr(self, k, v) for k, v in doc.data.iteritems()]

        for k, v in doc.links.iteritems():
            collection_type = Resource.registry.get(k, Resource)
            lazy_uri = wac.ResourceCollection(collection_type, utils.to_uri(v.url))
            setattr(self, k, lazy_uri)


class Client(wac.Client):
    config = wac_config

    def _serialize(self, data):
        data = json.dumps(data, default=self._deserialize)
        return 'application/json', data

    def _deserialize(self, response):
        if hasattr(response, "content"):
            data = json.loads(response.content)
        elif hasattr(response, "data"):
            data = response.data
        elif len(response) == 0:
            data = list()
        else:
            data = response.__dict__.copy()
            data.pop('uri', None)
            data = dict(
                (k, v)
                for k, v in data.iteritems()
                if not isinstance(v, (Resource, type(response).collection_cls))
            )
        return data


class JSONSchemaPage(wac.Page, ObjectifyMixin):
    pass


class Resource(wac.Resource, ObjectifyMixin):
    client = Client()
    registry = wac.ResourceRegistry()

    collection_cls = wac.ResourceCollection
    page_cls = JSONSchemaPage

    def verify_on(self, verification):
        if not hasattr(self, "verifications"):
            raise NameError("NotSupportVerifyMethod")
        attrs = verification.__dict__.copy()
        return self.verifications.create(attrs)


class ResourceErrors(RuntimeError, ObjectifyMixin):
    def __init__(self, **kwargs):
        self._objectify(self.__class__, **kwargs)
        if hasattr(self, "errors"):
            errors = []
            [errors.append(ResourceError(hal_codec=error)) for error in self.errors]
            self.errors = errors

    def __str__(self):
        return json.dumps(self, sort_keys=True, indent=2, default=lambda o: o.__dict__)


class ResourceError(Resource):
    type = "errors"



"""
Finix Python client library.
See ``README.md`` for usage advice.
"""

import os
import re

try:
    import setuptools
except ImportError:
    import distutils.core

    setup = distutils.core.setup
else:
    setup = setuptools.setup

PACKAGE = next((str(s) for s in setuptools.find_packages('.', exclude=("tests", "tests.*"))), None)
PWD = os.path.abspath(os.path.dirname(__file__))
VERSION = (
    re
        .compile(r".*__version__ = '(.*?)'", re.S)
        .match(open(os.path.join(PWD, PACKAGE, "__init__.py")).read())
        .group(1)
)

with open(os.path.join(PWD, "README.md")) as f:
    README = f.read()

requires = [
    "coreapi==1.20.0",
    "finix-wac==0.29"
]

extras_require = {
    "tests": [
        "coverage==4.0.3"
    ]
}

scripts = [
    # 'bin/citadel'
]

setup(
    name=PACKAGE,
    version=VERSION,
    url='https://finixpayments.com/',
    license='MIT License',
    author='Finix Payments',
    author_email='dev@finixpayments.com',
    description='Payments API',
    long_description=README,
    packages=[PACKAGE],
    test_suite='nose.collector',
    install_requires=requires,
    tests_require=extras_require['tests'],
    dependency_links=[],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    include_package_data=True,
    zip_safe=False,
    scripts=scripts,
    extras_require=extras_require,
    setup_requires=['nose>=1.3.7']
)
